# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import numpy as np
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{,
author="Jiang, Shengyi
and Fu, Sihui
and Lin, Nankai
and Fu, Yingwen",
title="Pre-trained Models and Evaluation Data for the Khmer Language",
year="2021",
publisher="Tsinghua Science and Technology",
}
"""

_DATASETNAME = "gklmip_newsclass"

_DESCRIPTION = """\
The GKLMIP Khmer News Dataset is scraped from the Voice of America Khmer website. \
The news articles in the dataset are categorized into 8 categories: culture, economics, education, \
environment, health, politics, rights and science.
"""

_HOMEPAGE = "https://github.com/GKLMIP/Pretrained-Models-For-Khmer"
_LANGUAGES = ["khm"]

_LICENSE = Licenses.UNKNOWN.value
_LOCAL = False

_URLS = {
    _DATASETNAME: "https://github.com/GKLMIP/Pretrained-Models-For-Khmer/raw/main/NewsDataset.zip",
}

_SUPPORTED_TASKS = [Tasks.TOPIC_MODELING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"

_TAGS = ["culture", "economic", "education", "environment", "health", "politics", "right", "science"]


class GklmipNewsclass(datasets.GeneratorBasedBuilder):
    """\
    The GKLMIP Khmer News Dataset is scraped from the Voice of America Khmer website. \
    The news articles in the dataset are categorized into 8 categories: culture, economics, education, \
    environment, health, politics, rights and science.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    SEACROWD_SCHEMA_NAME = "text"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "culture": datasets.Value("bool"),
                    "economic": datasets.Value("bool"),
                    "education": datasets.Value("bool"),
                    "environment": datasets.Value("bool"),
                    "health": datasets.Value("bool"),
                    "politics": datasets.Value("bool"),
                    "right": datasets.Value("bool"),
                    "science": datasets.Value("bool"),
                }
            )

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text_features(_TAGS)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "train.csv"),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "test.csv"),
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "dev.csv"),
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        dataset = pd.read_csv(filepath)
        reverse_encoding = dict(zip(range(len(_TAGS)), _TAGS))
        if self.config.schema == "source":
            for i, row in dataset.iterrows():
                yield i, {
                    "text": row["text"],
                    "culture": row["culture"],
                    "economic": row["economic"],
                    "education": row["education"],
                    "environment": row["environment"],
                    "health": row["health"],
                    "politics": row["politics"],
                    "right": row["right"],
                    "science": row["science"],
                }

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            for i, row in dataset.iterrows():
                yield i, {"id": i, "text": row["text"], "label": reverse_encoding[np.argmax(row[_TAGS])]}



import json
from pathlib import Path
from typing import List

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import (DEFAULT_SEACROWD_VIEW_NAME,
                                       DEFAULT_SOURCE_VIEW_NAME, Tasks)

_DATASETNAME = "ted_en_id"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["ind", "eng"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False
_CITATION = """\
@inproceedings{qi2018and,
  title={When and Why Are Pre-Trained Word Embeddings Useful for Neural Machine Translation?},
  author={Qi, Ye and Sachan, Devendra and Felix, Matthieu and Padmanabhan, Sarguna and Neubig, Graham},
  booktitle={Proceedings of the 2018 Conference of the North American Chapter of the Association for Computational Linguistics: Human Language Technologies, Volume 2 (Short Papers)},
  pages={529--535},
  year={2018}
}

@inproceedings{cahyawijaya-etal-2021-indonlg,
    title = "{I}ndo{NLG}: Benchmark and Resources for Evaluating {I}ndonesian Natural Language Generation",
    author = "Cahyawijaya, Samuel  and
      Winata, Genta Indra  and
      Wilie, Bryan  and
      Vincentio, Karissa  and
      Li, Xiaohong  and
      Kuncoro, Adhiguna  and
      Ruder, Sebastian  and
      Lim, Zhi Yuan  and
      Bahar, Syafri  and
      Khodra, Masayu  and
      Purwarianti, Ayu  and
      Fung, Pascale",
    booktitle = "Proceedings of the 2021 Conference on Empirical Methods in Natural Language Processing",
    month = nov,
    year = "2021",
    address = "Online and Punta Cana, Dominican Republic",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2021.emnlp-main.699",
    doi = "10.18653/v1/2021.emnlp-main.699",
    pages = "8875--8898",
    abstract = "Natural language generation (NLG) benchmarks provide an important avenue to measure progress and develop better NLG systems. Unfortunately, the lack of publicly available NLG benchmarks for low-resource languages poses a challenging barrier for building NLG systems that work well for languages with limited amounts of data. Here we introduce IndoNLG, the first benchmark to measure natural language generation (NLG) progress in three low-resource{---}yet widely spoken{---}languages of Indonesia: Indonesian, Javanese, and Sundanese. Altogether, these languages are spoken by more than 100 million native speakers, and hence constitute an important use case of NLG systems today. Concretely, IndoNLG covers six tasks: summarization, question answering, chit-chat, and three different pairs of machine translation (MT) tasks. We collate a clean pretraining corpus of Indonesian, Sundanese, and Javanese datasets, Indo4B-Plus, which is used to pretrain our models: IndoBART and IndoGPT. We show that IndoBART and IndoGPT achieve competitive performance on all tasks{---}despite using only one-fifth the parameters of a larger multilingual model, mBART-large (Liu et al., 2020). This finding emphasizes the importance of pretraining on closely related, localized languages to achieve more efficient learning and faster inference at very low-resource languages like Javanese and Sundanese.",
}
"""

_DESCRIPTION = """\
TED En-Id is a machine translation dataset containing Indonesian-English parallel sentences collected from the TED talk transcripts. We split the dataset and use 75% as the training set, 10% as the validation set, and 15% as the test set. Each of the datasets is evaluated in both directions, i.e., English to Indonesian (En → Id) and Indonesian to English (Id → En) translations.
"""

_HOMEPAGE = "https://github.com/IndoNLP/indonlg"

_LICENSE = "Creative Commons Attribution Share-Alike 4.0 International"

_URLs = {"indonlg": "https://storage.googleapis.com/babert-pretraining/IndoNLG_finals/downstream_task/downstream_task_datasets.zip"}

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class TEDEnId(datasets.GeneratorBasedBuilder):
    """TED En-Id is a machine translation dataset containing Indonesian-English parallel sentences collected from the TED talk transcripts."""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="ted_en_id_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="TED En-Id source schema",
            schema="source",
            subset_id="ted_en_id",
        ),
        SEACrowdConfig(
            name="ted_en_id_seacrowd_t2t",
            version=datasets.Version(_SEACROWD_VERSION),
            description="TED En-Id Nusantara schema",
            schema="seacrowd_t2t",
            subset_id="ted_en_id",
        ),
    ]

    DEFAULT_CONFIG_NAME = "ted_en_id_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features({"id": datasets.Value("string"), "text": datasets.Value("string"), "label": datasets.Value("string")})
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        base_path = Path(dl_manager.download_and_extract(_URLs["indonlg"])) / "IndoNLG_downstream_tasks" / "MT_TED_MULTI"
        data_files = {
            "train": base_path / "train_preprocess.json",
            "validation": base_path / "valid_preprocess.json",
            "test": base_path / "test_preprocess.json",
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_files["train"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": data_files["validation"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_files["test"]},
            ),
        ]

    def _generate_examples(self, filepath: Path):
        data = json.load(open(filepath, "r"))
        if self.config.schema == "source":
            for row in data:
                ex = {"id": row["id"], "text": row["text"], "label": row["label"]}
                yield row["id"], ex
        elif self.config.schema == "seacrowd_t2t":
            for row in data:
                ex = {
                    "id": row["id"],
                    "text_1": row["text"],
                    "text_2": row["label"],
                    "text_1_name": "eng",
                    "text_2_name": "ind",
                }
                yield row["id"], ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")


if __name__ == "__main__":
    datasets.load_dataset(__file__)



# coding=utf-8
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@InProceedings{nguyen2022viheathqa,
  author="Nguyen, Nhung Thi-Hong
  and Ha, Phuong Phan-Dieu
  and Nguyen, Luan Thanh
  and Van Nguyen, Kiet
  and Nguyen, Ngan Luu-Thuy",
  title="SPBERTQA: A Two-Stage Question Answering System Based on Sentence Transformers for Medical Texts",
  booktitle="Knowledge Science, Engineering and Management",
  year="2022",
  publisher="Springer International Publishing",
  address="Cham",
  pages="371--382",
  isbn="978-3-031-10986-7"
}
"""
_DATASETNAME = "vihealthqa"
_DESCRIPTION = """\
Vietnamese Visual Question Answering (ViVQA) consist of 10328 images and 15000 question-answer
pairs in Vietnamese for evaluating Vietnamese VQA models. This dataset is built based on 10328 randomly
selected images from MS COCO dataset. The question-answer pairs were based on the COCO-QA dataset that
was automatically translated from English to Vietnamese.
"""
_HOMEPAGE = "https://huggingface.co/datasets/tarudesu/ViHealthQA"
_LANGUAGES = ["vie"]
_LICENSE = Licenses.UNKNOWN.value
_PAPER_URL = "https://link.springer.com/chapter/10.1007/978-3-031-10986-7_30"
_LOCAL = False
_URLS = {
    "vihealthqa": {
        "train": "https://huggingface.co/datasets/tarudesu/ViHealthQA/raw/main/train.csv",
        "val": "https://huggingface.co/datasets/tarudesu/ViHealthQA/raw/main/val.csv",
        "test": "https://huggingface.co/datasets/tarudesu/ViHealthQA/raw/main/test.csv",
    }
}
_SUPPORTED_TASKS = [Tasks.QUESTION_ANSWERING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class ViHealthQADataset(datasets.GeneratorBasedBuilder):
    '''
This is a SeaCrowed dataloader for dataset Vietnamese Visual Question Answering (ViVQA), which consists of 10328 images and 15000 question-answer
pairs in Vietnamese for evaluating Vietnamese VQA models.
    '''
    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_qa",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_qa",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "answer": datasets.Value("string"),
                    "link": datasets.Value("string")
                }
            )
        elif self.config.schema == "seacrowd_qa":
            features = schemas.qa_features
            features["meta"] = {"link": datasets.Value("string")}
        else:
            raise ValueError(f"No schema matched for {self.config.schema}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS["vihealthqa"]
        data_dir = dl_manager.download_and_extract(urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_dir["val"],
                    "split": "val",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_dir["test"],
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        raw_examples = pd.read_csv(filepath)

        for eid, exam in raw_examples.iterrows():
            assert len(exam) == 4
            exam_id, exam_quest, exam_answer, exam_link = exam

            if self.config.schema == "source":
                yield eid, {"id": str(exam_id), "question": exam_quest, "answer": exam_answer, "link": exam_link}

            elif self.config.schema == "seacrowd_qa":
                yield eid, {
                    "id": str(eid),
                    "question_id": exam_id,
                    "document_id": str(eid),
                    "question": exam_quest,
                    "type": None,
                    "choices": [],
                    "context": exam_link,
                    "answer": [exam_answer],
                    "meta": {
                        "link": exam_link,
                    },
                }



import csv
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@article{galinato-etal-2023-context,
    title="Context-Based Profanity Detection and Censorship Using Bidirectional Encoder Representations from Transformers",
    author="Galinato, Valfrid and Amores, Lawrence and Magsino, Gino Ben and Sumawang, David Rafael",
    month="jan",
    year="2023"
    url="https://papers.ssrn.com/sol3/papers.cfm?abstract_id=4341604"
}
"""

_LOCAL = False
_LANGUAGES = ["tgl"]
_DATASETNAME = "tgl_profanity"
_DESCRIPTION = """\
This dataset contains 13.8k Tagalog sentences containing profane words, together
with binary labels denoting whether or not the sentence conveys profanity /
abuse / hate speech. The data was scraped from Twitter using a Python library
called SNScrape and annotated manually by a panel of native Filipino speakers.
"""

_HOMEPAGE = "https://huggingface.co/datasets/mginoben/tagalog-profanity-dataset/"
_LICENSE = Licenses.UNKNOWN.value
_SUPPORTED_TASKS = [Tasks.ABUSIVE_LANGUAGE_PREDICTION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"
_URLS = {
    "train": "https://huggingface.co/datasets/mginoben/tagalog-profanity-dataset/resolve/main/train.csv",
    "val": "https://huggingface.co/datasets/mginoben/tagalog-profanity-dataset/resolve/main/val.csv",
}


class TagalogProfanityDataset(datasets.GeneratorBasedBuilder):
    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "text"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"
    CLASS_LABELS = ["1", "0"]

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "label": datasets.Value("int64"),
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text_features(label_names=self.CLASS_LABELS)
        else:
            raise ValueError(f"Invalid config name: {self.config.schema}")
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        data_files = dl_manager.download_and_extract(_URLS)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_files["train"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": data_files["val"]},
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yield examples as (key, example) tuples"""
        with open(filepath, encoding="utf-8") as f:
            csv_reader = csv.reader(f, delimiter=",")
            next(csv_reader, None)  # skip the headers
            for idx, row in enumerate(csv_reader):
                text, label = row
                if self.config.schema == "source":
                    example = {"text": text, "label": int(label)}
                elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                    example = {"id": idx, "text": text, "label": int(label)}
                yield idx, example



import io

import conllu
import datasets

from seacrowd.utils.common_parser import load_ud_data_as_seacrowd_kb
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils import schemas
from seacrowd.utils.constants import DEFAULT_SEACROWD_VIEW_NAME, DEFAULT_SOURCE_VIEW_NAME, Licenses, Tasks

_DATASETNAME = "stb_ext"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["eng"]
_LOCAL = False
_CITATION = """\
@article{wang2019genesis,
title={From genesis to creole language: Transfer learning for singlish universal dependencies parsing and POS tagging},
author={Wang, Hongmin and Yang, Jie and Zhang, Yue},
journal={ACM Transactions on Asian and Low-Resource Language Information Processing (TALLIP)},
volume={19},
number={1},
pages={1--29},
year={2019},
publisher={ACM New York, NY, USA}
}
"""

_DESCRIPTION = """\
We adopt the Universal Dependencies protocol for constructing the Singlish dependency treebank, both as a new resource
for the low-resource languages and to facilitate knowledge transfer from English. Briefly, the STB-EXT dataset offers
a 3-times larger training set, while keeping the same dev and test sets from STB-ACL. We provide treebanks with both
gold-standard as well as automatically generated POS tags.
"""

_HOMEPAGE = "https://github.com/wanghm92/Sing_Par/tree/master/TALLIP19_dataset/treebank"

_LICENSE = Licenses.MIT.value

_PREFIX = "https://raw.githubusercontent.com/wanghm92/Sing_Par/master/TALLIP19_dataset/treebank/"
_URLS = {
    "gold_pos": {
        "train": _PREFIX + "gold_pos/train.ext.conll",
    },
    "en_ud_autopos": {"train": _PREFIX + "en-ud-autopos/en-ud-train.conllu.autoupos", "validation": _PREFIX + "en-ud-autopos/en-ud-dev.conllu.ann.auto.epoch24.upos", "test": _PREFIX + "en-ud-autopos/en-ud-test.conllu.ann.auto.epoch24.upos"},
    "auto_pos_multiview": {
        "train": _PREFIX + "auto_pos/multiview/train.autopos.multiview.conll",
        "validation": _PREFIX + "auto_pos/multiview/dev.autopos.multiview.conll",
        "test": _PREFIX + "auto_pos/multiview/test.autopos.multiview.conll",
    },
    "auto_pos_stack": {
        "train": _PREFIX + "auto_pos/stack/train.autopos.stack.conll",
        "validation": _PREFIX + "auto_pos/stack/dev.autopos.stack.conll",
        "test": _PREFIX + "auto_pos/stack/test.autopos.stack.conll",
    },
}
_POSTAGS = ["ADJ", "ADP", "ADV", "AUX", "CONJ", "DET", "INTJ", "NOUN", "NUM", "PART", "PRON", "PROPN", "PUNCT", "SCONJ", "SYM", "VERB", "X", "root"]
_SUPPORTED_TASKS = [Tasks.POS_TAGGING, Tasks.DEPENDENCY_PARSING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


def config_constructor(subset_id, schema, version):
    return SEACrowdConfig(name=f"{_DATASETNAME}_{subset_id}_{schema}",
                          version=datasets.Version(version), description=_DESCRIPTION,
                          schema=schema, subset_id=subset_id)


class StbExtDataset(datasets.GeneratorBasedBuilder):
    """This is a seacrowd dataloader for the STB-EXT dataset, which offers a 3-times larger training set, while keeping
    the same dev and test sets from STB-ACL. It provides treebanks with both gold-standard and automatically generated POS tags."""

    BUILDER_CONFIGS = [
        # source
        config_constructor(subset_id="auto_pos_stack", schema="source", version=_SOURCE_VERSION),
        config_constructor(subset_id="auto_pos_multiview", schema="source", version=_SOURCE_VERSION),
        config_constructor(subset_id="en_ud_autopos", schema="source", version=_SOURCE_VERSION),
        config_constructor(subset_id="gold_pos", schema="source", version=_SOURCE_VERSION),
        # seq_label
        config_constructor(subset_id="auto_pos_stack", schema="seacrowd_seq_label", version=_SEACROWD_VERSION),
        config_constructor(subset_id="auto_pos_multiview", schema="seacrowd_seq_label", version=_SEACROWD_VERSION),
        config_constructor(subset_id="en_ud_autopos", schema="seacrowd_seq_label", version=_SEACROWD_VERSION),
        config_constructor(subset_id="gold_pos", schema="seacrowd_seq_label", version=_SEACROWD_VERSION),
        # dependency parsing
        config_constructor(subset_id="auto_pos_stack", schema="seacrowd_kb", version=_SEACROWD_VERSION),
        config_constructor(subset_id="auto_pos_multiview", schema="seacrowd_kb", version=_SEACROWD_VERSION),
        config_constructor(subset_id="en_ud_autopos", schema="seacrowd_kb", version=_SEACROWD_VERSION),
        config_constructor(subset_id="gold_pos", schema="seacrowd_kb", version=_SEACROWD_VERSION),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_gold_pos_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    # metadata
                    "sent_id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "text_en": datasets.Value("string"),
                    # tokens
                    "id": [datasets.Value("string")],
                    "form": [datasets.Value("string")],
                    "lemma": [datasets.Value("string")],
                    "upos": [datasets.Value("string")],
                    "xpos": [datasets.Value("string")],
                    "feats": [datasets.Value("string")],
                    "head": [datasets.Value("string")],
                    "deprel": [datasets.Value("string")],
                    "deps": [datasets.Value("string")],
                    "misc": [datasets.Value("string")],
                }
            )
        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label_features(label_names=_POSTAGS)
        elif self.config.schema == "seacrowd_kb":
            features = schemas.kb_features
        else:
            raise ValueError(f"Invalid config: {self.config.schema}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """ "return splitGenerators"""
        urls = _URLS[self.config.subset_id]
        downloaded_files = dl_manager.download_and_extract(urls)
        splits = []
        if "train" in downloaded_files:
            splits.append(datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_files["train"]}))
        if "validation" in downloaded_files:
            splits.append(datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepath": downloaded_files["validation"]}))
        if "test" in downloaded_files:
            splits.append(datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": downloaded_files["test"]}))
        return splits

    def _generate_examples(self, filepath):
        def process_buffer(TextIO):
            BOM = "\ufeff"
            buffer = io.StringIO()
            for line in TextIO:
                line = line.replace(BOM, "") if BOM in line else line
                buffer.write(line)
            buffer.seek(0)
            return buffer

        with open(filepath, "r", encoding="utf-8") as data_file:
            tokenlist = list(conllu.parse_incr(process_buffer(data_file)))
            data_instances = []
            for idx, sent in enumerate(tokenlist):
                idx = sent.metadata["sent_id"] if "sent_id" in sent.metadata else idx
                tokens = [token["form"] for token in sent]
                txt = sent.metadata["text"] if "text" in sent.metadata else " ".join(tokens)
                example = {
                    # meta
                    "sent_id": str(idx),
                    "text": txt,
                    "text_en": txt,
                    # tokens
                    "id": [token["id"] for token in sent],
                    "form": [token["form"] for token in sent],
                    "lemma": [token["lemma"] for token in sent],
                    "upos": [token["upos"] for token in sent],
                    "xpos": [token["xpos"] for token in sent],
                    "feats": [str(token["feats"]) for token in sent],
                    "head": [str(token["head"]) for token in sent],
                    "deprel": [str(token["deprel"]) for token in sent],
                    "deps": [str(token["deps"]) for token in sent],
                    "misc": [str(token["misc"]) for token in sent]
                }
                data_instances.append(example)

            if self.config.schema == "source":
                pass
            if self.config.schema == "seacrowd_seq_label":
                data_instances = list(
                    map(
                        lambda d: {
                            "id": d["sent_id"],
                            "tokens": d["form"],
                            "labels": d["upos"],
                        },
                        data_instances,
                    )
                )
            if self.config.schema == "seacrowd_kb":
                data_instances = load_ud_data_as_seacrowd_kb(filepath, data_instances)
            for key, exam in enumerate(data_instances):
                yield key, exam



import pandas as pd
from pathlib import Path

def create_saintek_and_soshum_dataset(saintek_question: Path, soshum_question: Path, saintek_score: Path, soshum_score: Path):
    # creating dataframes
    saintek_question = pd.read_csv(saintek_question, sep=",", header="infer")
    saintek_score = pd.read_csv(saintek_score, sep=",", header="infer")
    soshum_question = pd.read_csv(soshum_question, sep=",", header="infer")
    soshum_score = pd.read_csv(soshum_score, sep=",", header="infer")

    # renaming
    saintek_question.rename(columns={'jawaban': 'kunci_jawaban'}, inplace='true')
    soshum_question.rename(columns={'jawaban': 'kunci_jawaban'}, inplace='true')

    # joining
    saintek_joined = pd.merge(saintek_question, saintek_score, how='left', on = 'type-problem')
    soshum_joined = pd.merge(soshum_question, soshum_score, how='left', on = 'type-problem')

    # adding answer key
    saintek_joined = pd.concat([saintek_joined, build_answer_key_score_dataframe(saintek_question)], ignore_index=True)
    soshum_joined = pd.concat([soshum_joined, build_answer_key_score_dataframe(soshum_question)], ignore_index=True)

    # re-case-ing (kebab to snake)
    saintek_joined.rename(columns={'type-problem': 'type_problem'}, inplace=True)
    soshum_joined.rename(columns={'type-problem': 'type_problem'}, inplace=True)

    # concatenate
    saintek_and_soshum = pd.concat([saintek_joined, soshum_joined], ignore_index=True).reset_index()

    return saintek_and_soshum


def build_answer_key_score_dataframe(question: pd.DataFrame):
    return pd.DataFrame({
        'type-problem' : question['type-problem'],
        'pertanyaan' : question['pertanyaan'],
        'kunci_jawaban' : question['kunci_jawaban'],
        'jawaban' : question['kunci_jawaban'],
        'score' : [5]*question.shape[0],
    })



"""
SEA Crowd Data Loader for SEA Wiki.
"""

import json
from itertools import product
from typing import Dict, List, Tuple

import datasets
from datasets import load_dataset
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@ONLINE{wikidump,
    author = "Wikimedia Foundation",
    title  = "Wikimedia Downloads",
    url    = "https://dumps.wikimedia.org"}
@ONLINE{wikipedia-hf,
    title  = "Huggingface Wikipedia Dataset",
    url    = "https://huggingface.co/datasets/wikipedia"}
@ONLINE{wikipedia-hf,
    title  = "Huggingface SEA Wikipedia Dataset",
    url    = "https://huggingface.co/datasets/sabilmakbar/sea_wiki"}
"""

logger = datasets.logging.get_logger(__name__)


with open(DownloadManager().download_and_extract("seacrowd/sea_datasets/sea_wiki/lang_config.json"), "r") as f:
    _LANG_CONFIG = json.load(f)

_LOCAL = False
_LANGUAGES = list(_LANG_CONFIG.keys())

_DATASETNAME = "sea_wiki"
_DESCRIPTION = """\
    SEA Lang & Local Langs Wikipedia Archives, dumped from WIkipedia HF and processed by boilerplate removal.
    This dataset consists of URL of referred Wikipedia Article, its Title, and its Text Data (Article Contents).
"""

_HOMEPAGE = "https://huggingface.co/datasets/sabilmakbar/sea_wiki"
_LICENSE = Licenses.CC_BY_SA_4_0.value

# url won't be used since it will implement load_dataset method on HF URL provided
_URL = "https://huggingface.co/datasets/sabilmakbar/sea_wiki"

_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING, Tasks.SUMMARIZATION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"

CONFIG_SUFFIXES_FOR_TASK = ["ssp", "t2t"]


def conform_init_config():
    """Assertion Function for Instantiated Configs"""
    if len(_LANGUAGES) == 0:
        raise AssertionError("No Languages detected from config!")
    if len(CONFIG_SUFFIXES_FOR_TASK) != len(_SUPPORTED_TASKS):
        raise AssertionError("Config prefixes doesn't matched in terms of `len` with `_SUPPORTED_TASKS`!")
    if len(CONFIG_SUFFIXES_FOR_TASK) == 0:
        raise AssertionError("Config prefixes and `_SUPPORTED_TASKS` have `len` of 0!")


conform_init_config()

# construct zipped arg for config instantiation
SCHEMA_PREFIX_AND_VERSION_PAIRS = list(zip(("source", "seacrowd"), (_SOURCE_VERSION, _SEACROWD_VERSION)))
CONFIG_NAME_AND_TASKS_PAIRS = list(zip(CONFIG_SUFFIXES_FOR_TASK, _SUPPORTED_TASKS))


def construct_configs(languages: list = None) -> List[SEACrowdConfig]:
    """
    The function `construct_configs` constructs a list of SEACrowdConfig objects based on the provided
    languages or a default language, and returns the list.

    input:
        languages (list, default None): The `languages` parameter is a list that specifies the languages for which the
        configurations need to be constructed. If no languages are provided (value=None), the first value in language config
        will be used.
    output:
        a list of `SEACrowdConfig` objects based on instantiated init variables
    """
    # set output var
    config_list = []

    # set default task for default config w/o task arg name (set to Tasks.SUMMARIZATION)
    _DEFAULT_TASK_IDX = [idx for idx, val in enumerate(_SUPPORTED_TASKS) if val == Tasks.SUMMARIZATION]

    # assert `_DEFAULT_TASK_IDX` to have len of 1
    if len(_DEFAULT_TASK_IDX) != 1:
        raise AssertionError("Unexpected `_DEFAULT_TASK` #item!")

    _DEFAULT_CONFIG_SUFFIX, _DEFAULT_TASK = list(CONFIG_NAME_AND_TASKS_PAIRS)[_DEFAULT_TASK_IDX[0]]

    # check `languages` variable and create config accordingly
    if languages is None:
        # set languages arg as list of first entry in `_LANGUAGES` if no lang arg received
        _languages = _LANGUAGES[0]

        config_list += [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{config_name_prefix}",
                version=datasets.Version(version),
                description=f"{_DATASETNAME} {config_name_prefix} schema for default task arg ({_DEFAULT_TASK})",
                schema=f"{config_name_prefix}_{_DEFAULT_CONFIG_SUFFIX}",
                subset_id=_languages,
            )
            for (config_name_prefix, version) in SCHEMA_PREFIX_AND_VERSION_PAIRS
        ]
        config_list += [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{config_name_prefix}_{config_name_suffix}",
                version=datasets.Version(version),
                description=f"{_DATASETNAME} {config_name_prefix} schema for {task_obj.name}",
                schema=f"{config_name_prefix}_{config_name_suffix}",
                subset_id=_languages,
            )
            for (config_name_prefix, version), (config_name_suffix, task_obj) in product(SCHEMA_PREFIX_AND_VERSION_PAIRS, CONFIG_NAME_AND_TASKS_PAIRS)
        ]

    # else, construct configs based on its lang
    else:
        for _LANG in languages:
            config_list += [
                SEACrowdConfig(
                    name=f"{_DATASETNAME}_{config_name_prefix}_{_LANG}_{config_name_suffix}",
                    version=datasets.Version(version),
                    description=f"{_DATASETNAME} {config_name_prefix} schema for {task_obj.name} and language code {_LANG}",
                    schema=f"{config_name_prefix}_{config_name_suffix}",
                    subset_id=_LANG,
                )
                for (config_name_prefix, version), (config_name_suffix, task_obj) in product(SCHEMA_PREFIX_AND_VERSION_PAIRS, CONFIG_NAME_AND_TASKS_PAIRS)
            ]

    return config_list


class SEAWikiDataset(datasets.GeneratorBasedBuilder):
    """SEA Wiki dataset from https://huggingface.co/datasets/sabilmakbar/sea_wiki"""

    # get all schema w/o lang arg + get all schema w/ lang arg
    BUILDER_CONFIGS = construct_configs() + construct_configs(_LANGUAGES)

    def _info(self) -> datasets.DatasetInfo:
        _config_schema_name = self.config.schema
        logger.info(f"Received schema name: {self.config.schema}")
        # self supervised training schema
        if CONFIG_SUFFIXES_FOR_TASK[0] in _config_schema_name:
            if "source" in _config_schema_name:
                features = datasets.Features({"url": datasets.Value("string"), "text": datasets.Value("string")})

            elif "seacrowd" in _config_schema_name:
                features = schemas.ssp_features

            else:
                raise ValueError(f"Unexpected schema received! {_config_schema_name}")

        # summarization schema
        elif CONFIG_SUFFIXES_FOR_TASK[1] in _config_schema_name:
            if "source" in _config_schema_name:
                features = datasets.Features({"url": datasets.Value("string"), "title": datasets.Value("string"), "text": datasets.Value("string")})

            elif "seacrowd" in _config_schema_name:
                features = schemas.text2text_features

            else:
                raise ValueError(f"Unexpected schema received! {_config_schema_name}")

        else:
            raise ValueError(f"Received unexpected config schema of {_config_schema_name}!")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        # args of dl_manager is a placeholder since this data loader will wrap the hf `load_dataset` from given _URL
        # directly using `_load_hf_data_from_remote`
        return [datasets.SplitGenerator(name=datasets.Split.TRAIN)]

    def _load_hf_data_from_remote(self):
        # construct remote_hf_reference by the last 2 of string-spliited of "/"
        _remote_hf_reference = "/".join(_URL.split("/")[-2:])
        _lang_args = _LANG_CONFIG[self.config.subset_id]["source_subset"]
        _split = "train"

        logger.info(f"Loading dataset from remote HF {_remote_hf_reference} with seacrowd lang args of {self.config.subset_id} and source lang args of {_lang_args} and split args of {_split}")
        _hf_dataset_source = load_dataset(_remote_hf_reference, lang=_lang_args, split=_split)

        return _hf_dataset_source

    def _generate_examples(self) -> Tuple[int, Dict]:

        _config_schema_name = self.config.schema
        loaded_data = self._load_hf_data_from_remote()

        # iterate over datapoints and arrange hf dataset schema in source to match w/ config args:
        for id_, _data in enumerate(loaded_data):
            if "source" in _config_schema_name:
                yield id_, {colname: _data[colname] for colname in self.info.features}

            # for ssp schema
            elif "seacrowd" in _config_schema_name and CONFIG_SUFFIXES_FOR_TASK[0] in _config_schema_name:
                yield id_, {"id": id_, "text": _data["text"]}

            # for summary schema
            elif "seacrowd" in _config_schema_name and CONFIG_SUFFIXES_FOR_TASK[1] in _config_schema_name:
                yield id_, {"id": id_, "text_1": _data["text"], "text_2": _data["title"], "text_1_name": "document", "text_2_name": "title"}

            else:
                raise ValueError(f"Received unexpected config schema of {_config_schema_name}!")



from pathlib import Path
from typing import List

import datasets
import pandas as pd
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

# No paper citation found.
_CITATION = """\
@article{bai2022training,
  title={Training a helpful and harmless assistant with reinforcement learning from human feedback},
  author={Bai, Yuntao and Jones, Andy and Ndousse, Kamal and Askell, Amanda and Chen, Anna and DasSarma, Nova and Drain, Dawn and Fort, Stanislav and Ganguli, Deep and Henighan, Tom and others},
  journal={arXiv preprint arXiv:2204.05862},
  year={2022}
}

"""

_LOCAL = False
_LANGUAGES = ["tha"]
_DATASETNAME = "thai_hh_rlhf"
_DESCRIPTION = """\
This is a Thai-translated dataset based on Anthropic/hh-rlhf using Google Cloud Translation. This repository
provides access to 161K Train dataset Anthropic/hh-rlhf (Thai-translated).

The original repository provides access to two different kinds of data:

1. Human preference data about helpfulness and harmlessness from Training a Helpful and Harmless Assistant with
Reinforcement Learning from Human Feedback. These data are meant to train preference (or reward) models for
subsequent RLHF training. These data are not meant for supervised training of dialogue agents. Training dialogue
agents on these data is likely to lead to harmful models and this shold be avoided.

2. Human-generated and annotated red teaming dialogues from Red Teaming Language Models to Reduce Harms: Methods,
Scaling Behaviors, and Lessons Learned. These data are meant to understand how crowdworkers red team models and
what types of red team attacks are succesful or not. The data are not meant for fine-tuning or preference modeling
(use the data above for preference modeling). These data are entire transcripts of conversations that are derived
from the harmlessness preference modeling data described above, where only the chosen response is incorporated into
the overall transcript. Furthermore, the transcripts are annotated with human and automated measurements of how
harmful the overall dialogues are.

The translated data corresponds to the first kind of data.
"""

_HOMEPAGE = "	https://huggingface.co/datasets/Thaweewat/hh-rlhf-th"
_LICENSE = Licenses.MIT.value
_URL = "https://huggingface.co/datasets/Thaweewat/hh-rlhf-th/resolve/main/hh-rlhf-train-161k-th.parquet"
_SUPPORTED_TASKS = [Tasks.REINFORCEMENT_LEARNING_WITH_HUMAN_FEEDBACK]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class ThaiHhRlhfDataset(datasets.GeneratorBasedBuilder):
    """Thai_HH_RLHF Dataset"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "text"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description="Thai_HH_RLHF source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description="Thai_HH_RLHF SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "chosen": datasets.Value("string"),
                    "rejected": datasets.Value("string"),
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text.features(label_names=["rejected", "chosen"])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        data_file = Path(dl_manager.download_and_extract(_URL))
        return [datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": data_file})]

    def _generate_examples(self, filepath: Path):
        """Yield examples as (key, example) tuples"""
        df = pd.read_parquet(filepath)
        for idx, row in df.iterrows():
            if self.config.schema == "source":
                example = {"chosen": row.get("chosen"), "rejected": row.get("rejected")}

                yield idx, example

            elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                for i, label in enumerate(["chosen", "rejected"]):
                    text = row.get(label)

                    example = {"id": str(idx), "text": text, "label": label}

                    yield (idx * 2) + i, example



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{van2018uit,
  title={UIT-VSFC: Vietnamese students’ feedback corpus for sentiment analysis},
  author={Van Nguyen, Kiet and Nguyen, Vu Duc and Nguyen, Phu XV and Truong, Tham TH and Nguyen, Ngan Luu-Thuy},
  booktitle={2018 10th international conference on knowledge and systems engineering (KSE)},
  pages={19--24},
  year={2018},
  organization={IEEE}
}
"""


_DATASETNAME = "uit_vsfc"

_DESCRIPTION = """\
This corpus consists of student feedback obtained from end-of-semester surveys at a Vietnamese university.
Feedback is classified into four possible topics: lecturer, curriculum, facility or others.
Feedback is also labeled as one of three sentiment polarities: positive, negative or neutral.
"""

_HOMEPAGE = "https://drive.google.com/drive/folders/1HooABJyrddVGzll7fgkJ6VzkG_XuWfRu"

_LANGUAGES = ["vie"]

_LICENSE = Licenses.UNKNOWN.value

_LOCAL = False


_URLS = {
    "train": {
        "sentences": "https://drive.google.com/uc?id=1nzak5OkrheRV1ltOGCXkT671bmjODLhP&export=download",
        "sentiments": "https://drive.google.com/uc?id=1ye-gOZIBqXdKOoi_YxvpT6FeRNmViPPv&export=download",
        "topics": "https://drive.google.com/uc?id=14MuDtwMnNOcr4z_8KdpxprjbwaQ7lJ_C&export=download",
    },
    "validation": {
        "sentences": "https://drive.google.com/uc?id=1sMJSR3oRfPc3fe1gK-V3W5F24tov_517&export=download",
        "sentiments": "https://drive.google.com/uc?id=1GiY1AOp41dLXIIkgES4422AuDwmbUseL&export=download",
        "topics": "https://drive.google.com/uc?id=1DwLgDEaFWQe8mOd7EpF-xqMEbDLfdT-W&export=download",
    },
    "test": {
        "sentences": "https://drive.google.com/uc?id=1aNMOeZZbNwSRkjyCWAGtNCMa3YrshR-n&export=download",
        "sentiments": "https://drive.google.com/uc?id=1vkQS5gI0is4ACU58-AbWusnemw7KZNfO&export=download",
        "topics": "https://drive.google.com/uc?id=1_ArMpDguVsbUGl-xSMkTF_p5KpZrmpSB&export=download",
    },
}

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS, Tasks.TOPIC_MODELING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class UITVSFCDataset(datasets.GeneratorBasedBuilder):
    """This corpus consists of student feedback obtained from end-of-semester surveys at a Vietnamese university.
    Feedback is classified into four possible topics: lecturer, curriculum, facility or others.
    Feedback is also labeled as one of three sentiment polarities: positive, negative or neutral."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SENTIMENT_LABEL_CLASSES = ["positive", "negative", "neutral"]
    TOPIC_LABEL_CLASSES = ["lecturer", "training_program", "others", "facility"]

    SEACROWD_SCHEMA_NAME = "text"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_sentiment_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_topic_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_sentiment_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_topic_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "sentence": datasets.Value("string"),
                    "sentiment": datasets.ClassLabel(names=self.SENTIMENT_LABEL_CLASSES),
                    "topic": datasets.ClassLabel(names=self.TOPIC_LABEL_CLASSES),
                }
            )
        elif self.config.name == f"{_DATASETNAME}_sentiment_seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text_features(self.SENTIMENT_LABEL_CLASSES)
        elif self.config.name == f"{_DATASETNAME}_topic_seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text_features(self.TOPIC_LABEL_CLASSES)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        data_dir = dl_manager.download(_URLS)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "sentences_path": data_dir["train"]["sentences"],
                    "sentiments_path": data_dir["train"]["sentiments"],
                    "topics_path": data_dir["train"]["topics"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "sentences_path": data_dir["test"]["sentences"],
                    "sentiments_path": data_dir["test"]["sentiments"],
                    "topics_path": data_dir["test"]["topics"],
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "sentences_path": data_dir["validation"]["sentences"],
                    "sentiments_path": data_dir["validation"]["sentiments"],
                    "topics_path": data_dir["validation"]["topics"],
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, sentences_path: Path, sentiments_path: Path, topics_path: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        if self.config.schema == "source":
            with open(sentences_path, encoding="utf-8") as sentences, open(sentiments_path, encoding="utf-8") as sentiments, open(topics_path, encoding="utf-8") as topics:
                for key, (sentence, sentiment, topic) in enumerate(zip(sentences, sentiments, topics)):
                    yield key, {
                        "sentence": sentence.strip(),
                        "sentiment": int(sentiment.strip()),
                        "topic": int(topic.strip()),
                    }

        elif self.config.name == f"{_DATASETNAME}_sentiment_seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            with open(sentences_path, encoding="utf-8") as sentences, open(sentiments_path, encoding="utf-8") as sentiments:
                for key, (sentence, sentiment) in enumerate(zip(sentences, sentiments)):
                    yield key, {"id": str(key), "text": sentence.strip(), "label": int(sentiment.strip())}
        elif self.config.name == f"{_DATASETNAME}_topic_seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            with open(sentences_path, encoding="utf-8") as sentences, open(topics_path, encoding="utf-8") as topics:
                for key, (sentence, topic) in enumerate(zip(sentences, topics)):
                    yield key, {
                        "id": str(key),
                        "text": sentence.strip(),
                        "label": int(topic.strip()),
                    }



import csv
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import SCHEMA_TO_FEATURES, Licenses, Tasks

_CITATION = """\
@inproceedings{lin2022fewshot,
    author       = {Xi Victoria Lin and
                    Todor Mihaylov and
                    Mikel Artetxe and
                    Tianlu Wang and
                    Shuohui Chen and
                    Daniel Simig and
                    Myle Ott and
                    Naman Goyal and
                    Shruti Bhosale and
                    Jingfei Du and
                    Ramakanth Pasunuru and
                    Sam Shleifer and
                    Punit Singh Koura and
                    Vishrav Chaudhary and
                    Brian O'Horo and
                    Jeff Wang and
                    Luke Zettlemoyer and
                    Zornitsa Kozareva and
                    Mona T. Diab and
                    Veselin Stoyanov and
                    Xian Li},
    editor       = {Yoav Goldberg and
                    Zornitsa Kozareva and
                    Yue Zhang},
    title        = {Few-shot Learning with Multilingual Generative Language Models},
    booktitle    = {Proceedings of the 2022 Conference on Empirical Methods in Natural
                    Language Processing, {EMNLP} 2022, Abu Dhabi, United Arab Emirates,
                    December 7-11, 2022},
    pages        = {9019--9052},
    publisher    = {Association for Computational Linguistics},
    year         = {2022},
    url          = {https://doi.org/10.18653/v1/2022.emnlp-main.616},
    doi          = {10.18653/V1/2022.EMNLP-MAIN.616},
}
"""

_DATASETNAME = "xstorycloze"
_DESCRIPTION = """\
XStoryCloze consists of the professionally translated version of the English StoryCloze
dataset (Spring 2016 version) to 10 non-English languages. This dataset is released by
Meta AI.
"""
_HOMEPAGE = "https://huggingface.co/datasets/juletxara/xstory_cloze"
_LANGUAGES = ["ind", "mya"]
_LICENSE = Licenses.CC_BY_SA_4_0.value

_LOCAL = False
_BASE_URL = "https://huggingface.co/datasets/juletxara/xstory_cloze/resolve/main/spring2016.val.{lang}.tsv.split_20_80_{split}.tsv"
_SUPPORTED_TASKS = [Tasks.COMMONSENSE_REASONING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class XStoryClozeDataset(datasets.GeneratorBasedBuilder):
    """XStoryCloze subset for Indonesian and Burmese language."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SUBSET = ["id", "my"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{subset}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} {subset} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}_{subset}",
        )
        for subset in SEACROWD_SUBSET
    ] + [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{subset}_seacrowd_qa",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} {subset} SEACrowd schema",
            schema="seacrowd_qa",
            subset_id=f"{_DATASETNAME}_{subset}",
        )
        for subset in SEACROWD_SUBSET
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_{SEACROWD_SUBSET[0]}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "story_id": datasets.Value("string"),
                    "input_sentence_1": datasets.Value("string"),
                    "input_sentence_2": datasets.Value("string"),
                    "input_sentence_3": datasets.Value("string"),
                    "input_sentence_4": datasets.Value("string"),
                    "sentence_quiz1": datasets.Value("string"),
                    "sentence_quiz2": datasets.Value("string"),
                    "answer_right_ending": datasets.Value("int32"),
                }
            )
        elif self.config.schema == "seacrowd_qa":
            features = SCHEMA_TO_FEATURES["QA"]

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        lang = self.config.name.split("_")[1]
        filepaths = dl_manager.download_and_extract(
            {
                "train": _BASE_URL.format(lang=lang, split="train"),
                "test": _BASE_URL.format(lang=lang, split="eval"),
            }
        )

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": filepaths["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": filepaths["test"],
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        with open(filepath, encoding="utf-8") as f:
            data = csv.reader(f, quotechar='"', delimiter="\t", quoting=csv.QUOTE_ALL, skipinitialspace=True)
            _ = next(data)  # skip header
            if self.config.schema == "source":
                for id, row in enumerate(data):
                    yield id, {
                        "story_id": row[0],
                        "input_sentence_1": row[1],
                        "input_sentence_2": row[2],
                        "input_sentence_3": row[3],
                        "input_sentence_4": row[4],
                        "sentence_quiz1": row[5],
                        "sentence_quiz2": row[6],
                        "answer_right_ending": int(row[7]),
                    }
            elif self.config.schema == "seacrowd_qa":
                for id, row in enumerate(data):
                    question = " ".join(row[1:5])
                    choices = [row[5], row[6]]
                    yield id, {
                        "id": str(id),
                        "question_id": row[0],
                        "document_id": None,
                        "question": question,
                        "type": "multiple_choice",
                        "choices": choices,
                        "context": None,
                        "answer": [choices[int(row[7]) - 1]],
                        "meta": {},
                    }



import os
from pathlib import Path
from typing import Dict, List, Tuple
from seacrowd.utils.constants import Tasks
from seacrowd.utils import schemas

import datasets
import json
import xml.etree.ElementTree as ET

from seacrowd.utils.configs import SEACrowdConfig

_CITATION = """\
@INPROCEEDINGS{8074648,
  author={Suherik, Gilang Julian and Purwarianti, Ayu},
  booktitle={2017 5th International Conference on Information and Communication Technology (ICoIC7)}, 
  title={Experiments on coreference resolution for Indonesian language with lexical and shallow syntactic features}, 
  year={2017},
  volume={},
  number={},
  pages={1-5},
  doi={10.1109/ICoICT.2017.8074648}}
"""

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_DATASETNAME = "id_coreference_resolution"

_DESCRIPTION = """\
We built Indonesian coreference resolution that solves not only pronoun referenced to proper noun, but also proper noun to proper noun and pronoun to pronoun.
The differences with the available Indonesian coreference resolution lay on the problem scope and features. 
We conducted experiments using various features (lexical and shallow syntactic features) such as appositive feature, nearest candidate feature, direct sentence feature, previous and next word feature, and a lexical feature of first person. 
We also modified the method to build the training set by selecting the negative examples by cross pairing every single markable that appear between antecedent and anaphor. 
Compared with two available methods to build the training set, we conducted experiments using C45 algorithm. 
Using 200 news sentences, the best experiment achieved 71.6% F-Measure score.
"""

_HOMEPAGE = "https://github.com/tugas-akhir-nlp/indonesian-coreference-resolution-cnn/tree/master/data"

_LICENSE = "Creative Commons Attribution-ShareAlike 4.0"

_URLS = {
    _DATASETNAME: {
        "train": "https://raw.githubusercontent.com/tugas-akhir-nlp/indonesian-coreference-resolution-cnn/master/data/training/data.xml",
        "test": "https://raw.githubusercontent.com/tugas-akhir-nlp/indonesian-coreference-resolution-cnn/master/data/testing/data.xml"
    }
}

_SUPPORTED_TASKS = [Tasks.COREFERENCE_RESOLUTION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

class IDCoreferenceResolution(datasets.GeneratorBasedBuilder):

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="id_coreference_resolution_source",
            version=SOURCE_VERSION,
            description="ID Coreference Resolution source schema",
            schema="source",
            subset_id="id_coreference_resolution",
        ),
        SEACrowdConfig(
            name="id_coreference_resolution_seacrowd_kb",
            version=SEACROWD_VERSION,
            description="ID Coreference Resolution Nusantara schema",
            schema="seacrowd_kb",
            subset_id="id_coreference_resolution",
        ),
    ]

    DEFAULT_CONFIG_NAME = "id_coreference_resolution_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "phrases": [
                        {
                            "id": datasets.Value("string"),
                            "type": datasets.Value("string"),
                            "text": [
                                {
                                    "word": datasets.Value("string"),
                                    "ne": datasets.Value("string"),
                                    "label": datasets.Value("string")
                                }
                            ]
                        }
                    ]
                }
            )

        elif self.config.schema == "seacrowd_kb":
            features = schemas.kb_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]

        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_dir["test"],
                    "split": "test",
                },
            ),
        ]

    def _parse_phrase(self, phrase):
        splitted_text = phrase.text.split(" ")
        splitted_ne = []
        if ("ne" in phrase.attrib):
            splitted_ne = phrase.attrib["ne"].split("|")
        words = []
        for i in range(0, len(splitted_text)):
            word = splitted_text[i].split("\\")
            ne = ""
            label = ""
            if (i < len(splitted_ne)):
                ne = splitted_ne[i]
            if (len(word) > 1):
                label = word[1]
            words.append({
                "word": word[0],
                "ne": ne,
                "label": label
            })
        
        id = ""

        if ("id" in phrase.attrib):
            id = phrase.attrib["id"]

        return {
            "id": id,
            "type": phrase.attrib["type"],
            "text": words
        }


    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        data = ET.parse(filepath).getroot()

        for each_sentence in data:
            sentence = {
                "id": each_sentence.attrib["id"],
                "phrases": [],
            }
            for phrase in each_sentence:
                parsed_phrase = self._parse_phrase(phrase)
                sentence["phrases"].append(parsed_phrase)

            if self.config.schema == "source":
                yield int(each_sentence.attrib["id"]), sentence

            elif self.config.schema == "seacrowd_kb":
                ex = {
                    "id": each_sentence.attrib["id"],
                    "passages": [],
                    "entities": [
                        {
                            "id": phrase["id"],
                            "type": phrase["type"],
                            "text": [text["word"] for text in phrase["text"]],
                            "offsets": [[0, len(text["word"])] for text in phrase["text"]],
                            "normalized": [{
                                "db_name": text["ne"],
                                "db_id": ""
                            } for text in phrase["text"]],
                        }
                        for phrase in sentence["phrases"]
                    ],
                    "coreferences": [
                        {
                            "id": each_sentence.attrib["id"],
                            "entity_ids": [phrase["id"] for phrase in sentence["phrases"]]
                        }
                    ],
                    "events": [],
                    "relations": [],
                }
                yield int(each_sentence.attrib["id"]), ex



"""
SEA Crowd Data Loader for Fleurs.
"""

import json
from itertools import product
from typing import Dict, List, Tuple

import datasets
import pathlib
from datasets import load_dataset
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import TASK_TO_SCHEMA, Licenses, Tasks

_CITATION = """
@inproceedings{conneau22_interspeech,
  author={Alexis Conneau and Ankur Bapna and Yu Zhang and Min Ma and Patrick {von Platen} and Anton Lozhkov and Colin Cherry
    and Ye Jia and Clara Rivera and Mihir Kale and Daan {van Esch} and Vera Axelrod and Simran Khanuja and Jonathan Clark
    and Orhan Firat and Michael Auli and Sebastian Ruder and Jason Riesa and Melvin Johnson},
  title={{XTREME-S: Evaluating Cross-lingual Speech Representations}},
  year=2022,
  booktitle={Proc. Interspeech 2022},
  pages={3248--3252},
  doi={10.21437/Interspeech.2022-10007}
}
"""

logger = datasets.logging.get_logger(__name__)


_LOCAL = False

# since this fleurs source already subsets SEA langs, the names on lang group id is hard-coded
_LANG_GROUP_ID = ["south_east_asian_sea"]

_DATASETNAME = "fleurs"

path_to_here = pathlib.Path(__file__).parent.absolute()
with open(DownloadManager().download_and_extract(f"{path_to_here}/lang_config.json"), "r") as f:
    _LANG_CONFIG = json.load(f)
_LANGUAGES = list(_LANG_CONFIG.keys())

_DESCRIPTION = """\
    Fleurs dataset is a part of XTREME-S benchmark to evaluate universal cross-lingual speech representations in many languages.
    Fleurs is used for two tasks: automatic speech recognition and speech classification.
    Fleurs covers 10 language native to Southeast Asian and other 3 major languages
    mostly spoken in few of Southeast Asia countries (Mandarin Chinese, Portuguese, and Tamil).
"""

_HOMEPAGE = "https://huggingface.co/datasets/google/xtreme_s"
_LICENSE = Licenses.CC_BY_4_0.value

# url won't be used since it will implement load_dataset method on HF URL provided
_URL = "https://huggingface.co/datasets/google/xtreme_s"

# construct remote_hf_reference by the last 2 of string-spliited of "/" (expected: "google/xtreme_s")
_HF_REMOTE_REF = "/".join(_URL.split("/")[-2:])

_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION, Tasks.SPEECH_LANGUAGE_IDENTIFICATION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"

CONFIG_SUFFIXES_FOR_TASK = [TASK_TO_SCHEMA.get(task).lower() for task in _SUPPORTED_TASKS]


def conform_init_config():
    """Assertion Function for Instantiated Configs"""
    if len(_LANGUAGES) == 0:
        raise AssertionError("No Languages detected from config!")
    if len(CONFIG_SUFFIXES_FOR_TASK) != len(_SUPPORTED_TASKS):
        raise AssertionError("Config prefixes doesn't matched in terms of `len` with `_SUPPORTED_TASKS`!")
    if len(CONFIG_SUFFIXES_FOR_TASK) == 0:
        raise AssertionError("Config prefixes and `_SUPPORTED_TASKS` have `len` of 0!")


conform_init_config()


def construct_configs_on_langs(languages: list = None) -> List[SEACrowdConfig]:
    """
    The function `construct_configs` constructs a list of SEACrowdConfig objects based on the provided
    languages or a default language, and returns the list.

    input:
        languages (list): The `languages` parameter is a list that specifies the languages for which the
        configurations need to be constructed. If no languages are provided (value=None), the first value in language config
        will be used.
    output:
        a list of `SEACrowdConfig` objects based on instantiated init variables
    """
    # set output var
    config_list = []

    # set flag whether the task is lang-agnostic based on extended `_SUPPORTED_TASKS`
    IS_TASK_LANG_SUBSETTED = [True, False]

    TASKS_AND_CONFIG_SUFFIX_PAIRS = list(zip(_SUPPORTED_TASKS, CONFIG_SUFFIXES_FOR_TASK, IS_TASK_LANG_SUBSETTED))

    # implement source schema
    version, config_name_prefix = _SOURCE_VERSION, "source"
    config_list += [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{_LANG}_{config_name_prefix}",
            version=datasets.Version(version),
            description=f"{_DATASETNAME} {config_name_prefix} schema for language code {_LANG}",
            schema=f"{config_name_prefix}",
            subset_id=_LANG,
        )
        for _LANG in languages
    ]

    # implement SEACrowd schema
    version, config_name_prefix = _SEACROWD_VERSION, "seacrowd"
    for (task_obj, config_name_suffix, is_lang_subsetted) in TASKS_AND_CONFIG_SUFFIX_PAIRS:
        if is_lang_subsetted:
            # construct configs based on its lang, since the task & config needs to defined per lang
            # for this dataloader, Tasks.SPEECH_RECOGNITION will enter this condition
            config_list += [
                SEACrowdConfig(
                    name=f"{_DATASETNAME}_{_LANG}_{config_name_prefix}_{config_name_suffix}",
                    version=datasets.Version(version),
                    description=f"{_DATASETNAME} {config_name_prefix} schema for {task_obj.name} and language code {_LANG}",
                    schema=f"{config_name_prefix}_{config_name_suffix}",
                    subset_id=_LANG,
                )
                for _LANG in languages
            ]

        else:
            # else, its defined for all languages
            # for this dataloader, Tasks.SPEECH_LANGUAGE_IDENTIFICATION will enter this condition
            # however no "source" schema will be defined here (the source will follow this `fleurs_{lang_code}_source` config name)
            config_list.append(
                SEACrowdConfig(
                    name=f"{_DATASETNAME}_{config_name_prefix}_{config_name_suffix}",
                    version=datasets.Version(version),
                    description=f"{_DATASETNAME} {config_name_prefix} schema for {task_obj.name}",
                    schema=f"{config_name_prefix}_{config_name_suffix}",
                    subset_id="all",
                )
            )

    return config_list


class FleursDataset(datasets.GeneratorBasedBuilder):
    """Fleurs dataset from https://huggingface.co/datasets/google/xtreme_s"""

    # get all schema w/o lang arg + get all schema w/ lang arg
    BUILDER_CONFIGS = construct_configs_on_langs(_LANGUAGES)

    def _info(self) -> datasets.DatasetInfo:
        _config_schema_name = self.config.schema
        logger.info(f"Received schema name: {self.config.schema}")

        # source schema
        if _config_schema_name == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("int32"),
                    "num_samples": datasets.Value("int32"),
                    "path": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16_000),
                    "transcription": datasets.Value("string"),
                    "raw_transcription": datasets.Value("string"),
                    "gender": datasets.ClassLabel(names=["male", "female", "other"]),
                    "lang_id": datasets.ClassLabel(names=_LANGUAGES),
                    "language": datasets.Value("string"),
                    "lang_group_id": datasets.ClassLabel(
                        names=_LANG_GROUP_ID)
                }
            )

        # asr transcription schema for seacrowd
        elif _config_schema_name == f"seacrowd_{CONFIG_SUFFIXES_FOR_TASK[0]}":
            features = schemas.speech_text_features

        # speech lang classification schema for seacrowd
        elif _config_schema_name == f"seacrowd_{CONFIG_SUFFIXES_FOR_TASK[1]}":
            features = schemas.speech_features(label_names=_LANGUAGES)

        else:
            raise ValueError(f"Unexpected schema received! {_config_schema_name}")

        return datasets.DatasetInfo(description=_DESCRIPTION, features=features, homepage=_HOMEPAGE, license=_LICENSE, citation=_CITATION)

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        # args of dl_manager is useless since this data loader will wrap the hf `load_dataset` from given _URL
        return [
            datasets.SplitGenerator(
                name=split_name,
                gen_kwargs={"split_name": split_name._name})
            for split_name in (
                datasets.Split.TRAIN,
                datasets.Split.VALIDATION,
                datasets.Split.TEST)
        ]

    def _load_hf_data_from_remote(self, split_name: str) -> datasets.DatasetDict:

        if self.config.subset_id == "all":
            raise ValueError("Unexpected subset_id value of `all` received in eager-load of SEACrowd fleurs loader!")
        else:
            _config_name_args = "fleurs." + _LANG_CONFIG[self.config.subset_id]["fleurs_lang_code"] + "_" + _LANG_CONFIG[self.config.subset_id]["fleurs_country_code"]

        logger.info(f"Loading dataset from remote HF {_HF_REMOTE_REF} with seacrowd lang args of {self.config.subset_id} and hf-source config args of {_config_name_args}")
        _hf_dataset_source = load_dataset(_HF_REMOTE_REF, _config_name_args, split=split_name)

        return _hf_dataset_source

    def _lazy_load_hf_data_from_remote(self, split_name: str) -> datasets.DatasetDict:

        if self.config.subset_id != "all":
            raise ValueError(f"Unexpected subset_id value of {self.config.subset_id} received in lazy-load of SEACrowd fleurs loader!")
        else:
            _config_name_args = [(f"fleurs.{fleurs_lang_info['fleurs_lang_code']}_{fleurs_lang_info['fleurs_country_code']}", lang) for lang, fleurs_lang_info in _LANG_CONFIG.items()]

        for _config, lang_name in _config_name_args:
            logger.info(f"Loading dataset from remote HF {_HF_REMOTE_REF} with seacrowd lang args of {self.config.subset_id} and hf-source config args of {_config}")
            yield load_dataset(_HF_REMOTE_REF, _config, split=split_name), lang_name

    def _generate_examples(self, split_name: str) -> Tuple[int, Dict]:

        _config_schema_name = self.config.schema

        # for source schema and asr transcription schema (the data is loaded eagerly, since it's splitted by lang)
        if _config_schema_name in ("source", f"seacrowd_{CONFIG_SUFFIXES_FOR_TASK[0]}"):
            loaded_data = self._load_hf_data_from_remote(split_name)

            # iterate over datapoints and arrange hf dataset schema in source to match w/ config args:
            for id_, _data in enumerate(loaded_data):
                if _config_schema_name == "source":

                    #re-map "language_id" and "lang_group_id"
                    _data["lang_id"] = _LANGUAGES.index(self.config.subset_id)
                    _data["lang_group_id"] = 0

                    yield id_, {
                        colname: _data[colname] for colname in self.info.features}

                # 2 notes on seacrowd schema for ASR:
                # 1. since in source data, no speakers id nor its info were provided, it will be filled by default values:
                #    ("" for any data string-typed, and -1 for age data int-typed)
                # 2. the "id" is re-created on sequential order on loaded data bcs it's original id
                #    doesn't pass unit-test of seacrowd schema

                elif "seacrowd" in _config_schema_name:
                    yield id_, {
                        "id": id_,
                        "path": _data["path"],
                        "audio": _data["audio"],
                        "text": _data["transcription"],
                        "speaker_id": "",
                        "metadata": {
                            "speaker_age": -1,
                            "speaker_gender": _data["gender"],
                        },
                    }

                else:
                    raise ValueError(f"Received unexpected config schema of {_config_schema_name}!")

                # add id_ so it will be globally unique
                id_ += 1

        # for speech lang classification schema (the data is loaded lazily per lang)
        elif _config_schema_name == f"seacrowd_{CONFIG_SUFFIXES_FOR_TASK[1]}":
            loaded_data = self._lazy_load_hf_data_from_remote(split_name)
            id_ = 0
            while True:
                _loaded_data, lang_info = next(loaded_data, (None, None))
                if _loaded_data is None:
                    break
                # iterate over datapoints and arrange hf dataset schema in source to match w/ config args:
                for _data in _loaded_data:
                    yield id_, {
                        "id": id_,
                        "path": _data["path"],
                        "audio": _data["audio"],
                        "labels": _LANGUAGES.index(lang_info),
                        "speaker_id": "",
                        "metadata": {
                            "speaker_age": -1,
                            "speaker_gender": _data["gender"],
                        },
                    }

                    # add id_ so it will be globally unique
                    id_ += 1

        else:
            raise ValueError(f"Received unexpected config schema of {_config_schema_name}!")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{longpre-etal-2021-mkqa,
    title = "{MKQA}: A Linguistically Diverse Benchmark for Multilingual Open Domain Question Answering",
    author = "Longpre, Shayne  and
      Lu, Yi  and
      Daiber, Joachim",
    editor = "Roark, Brian  and
      Nenkova, Ani",
    journal = "Transactions of the Association for Computational Linguistics",
    volume = "9",
    year = "2021",
    address = "Cambridge, MA",
    publisher = "MIT Press",
    url = "https://aclanthology.org/2021.tacl-1.82",
    doi = "10.1162/tacl_a_00433",
    pages = "1389--1406",
}
"""

_DATASETNAME = "mkqa"

_DESCRIPTION = """\
Multilingual Knowledge Questions and Answers (MKQA), an open-domain question answering evaluation set comprising 10k question-answer pairs aligned across 26 typologically diverse languages (260k question-answer pairs in total)
"""

_HOMEPAGE = "https://github.com/apple/ml-mkqa"

_LICENSE = Licenses.CC_BY_SA_3_0.value

_LOCAL = False

_URLS = {
    _DATASETNAME: "https://github.com/apple/ml-mkqa/raw/main/dataset/mkqa.jsonl.gz",
}

_SUPPORTED_TASKS = [Tasks.QUESTION_ANSWERING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_LANGUAGES = [
    "khm",
    "zsm",
    "tha",
    "vie",
]  # follows the convention of 3-letter code as suggested since NusaCrowd.


class MKQADataset(datasets.GeneratorBasedBuilder):
    """
    MKQA, an open-domain question answering evaluation set comprising 10k question-answer pairs
    aligned across 26 typologically diverse languages (260k question-answer pairs in total).
    The goal of this dataset is to provide a challenging benchmark for question answering quality
    across a wide set of languages.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    _ANS_TYPES = [
        "binary",
        "date",
        "entity",
        "long_answer",
        "number",
        "number_with_unit",
        "short_phrase",
        "unanswerable",
    ]

    _SOURCE_LANGUAGES = [
        "km",
        "ms",
        "th",
        "vi",
        # Filtered out:
        # "ar", "da", "de", "en", "es", "fi", "fr", "he", "hu", "it", "ja", "ko",
        # "nl", "no", "pl", "pt", "ru", "sv", "tr", "zh_cn", "zh_hk", "zh_tw",
    ]

    _LANG_3TO2 = {
        "khm": "km",
        "zsm": "ms",
        "tha": "th",
        "vie": "vi",
    }

    BUILDER_CONFIGS = [
        *[
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{subset_lang}{'_' if subset_lang else ''}source",
                version=datasets.Version(_SOURCE_VERSION),
                description=f"{_DATASETNAME} source schema",
                schema="source",
                subset_id=f"{_DATASETNAME}_{subset_lang}",
            )
            for subset_lang in ["", *_LANGUAGES]
        ],
        *[
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{subset_lang}{'_' if subset_lang else ''}seacrowd_qa",
                version=datasets.Version(_SEACROWD_VERSION),
                description=f"{_DATASETNAME} SEACrowd schema",
                schema="seacrowd_qa",
                subset_id=f"{_DATASETNAME}_{subset_lang}",
            )
            for subset_lang in ["", *_LANGUAGES]
        ],
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        lang = self.config.subset_id.rsplit("_", 1)[-1]
        lang = self._LANG_3TO2.get(lang, lang)

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "query": datasets.Value("string"),
                    "answers": {
                        cur_lang: [
                            {
                                "type": datasets.ClassLabel(names=self._ANS_TYPES),
                                "entity": datasets.Value("string"),
                                "text": datasets.Value("string"),
                                "aliases": [datasets.Value("string")],
                            }
                        ]
                        for cur_lang in ([lang] if lang else self._SOURCE_LANGUAGES)
                    },
                    "queries": {cur_lang: datasets.Value("string") for cur_lang in ([lang] if lang else self._SOURCE_LANGUAGES)},
                    "example_id": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_qa":
            features = schemas.qa_features
            features["meta"]["answer_entity"] = datasets.Sequence(datasets.Value("string"))
            features["meta"]["answer_aliases"] = datasets.Sequence(datasets.Sequence(datasets.Value("string")))
            features["meta"]["answer_type"] = datasets.Sequence(datasets.ClassLabel(names=self._ANS_TYPES))

        else:  # schema not found! should NOT reach here ...
            raise NotImplementedError()

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        data_path = dl_manager.download_and_extract(urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_path},
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        lang = self.config.subset_id.rsplit("_", 1)[-1]
        lang = self._LANG_3TO2.get(lang, lang)

        datas = []
        with open(filepath, "r", encoding="utf8") as ipt:
            for cur in map(json.loads, ipt):
                cur["example_id"] = str(cur["example_id"])
                for key in ["answers", "queries"]:
                    cur[key] = {k: v for k, v in cur[key].items() if k in ([lang] if lang else self._SOURCE_LANGUAGES)}
                datas.append(cur)

        if self.config.schema == "source":
            for cur in datas:
                for anslist in cur["answers"].values():
                    for ans in anslist:
                        ans.setdefault("entity", "")
                        ans.setdefault("aliases", [])
                yield int(cur["example_id"]), cur

        elif self.config.schema == "seacrowd_qa":
            for cur in datas:
                for cur_lang in [lang] if lang else map(lambda k: self._LANG_3TO2.get(k, k), _LANGUAGES):
                    ret = {
                        "id": f'{cur["example_id"]}_{cur_lang}',
                        "question_id": cur["example_id"],
                        "document_id": "",
                        "question": cur["queries"][cur_lang],
                        "type": "open_domain",
                        "choices": [],
                        "context": "",
                        "answer": [ans.get("text", None) for ans in cur["answers"][cur_lang]],
                        "meta": {f"answer_{k}": [ans.get(k, None) for ans in cur["answers"][cur_lang]] for k in ["entity", "aliases", "type"]},
                    }
                    ret["meta"]["answer_aliases"] = list(map(lambda a: [] if a is None else a, ret["meta"]["answer_aliases"]))
                    yield ret["id"], ret



import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
from pathlib import Path
from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, Licenses

_CITATION = """\
@misc{united1998universal,
    title={The Universal Declaration of Human Rights, 1948-1998},
    author={United Nations},
    year={1998},
    publisher={United Nations Dept. of Public Information New York}
}
"""
_DATASETNAME = "udhr"
_DESCRIPTION = """\
The Universal Declaration of Human Rights (UDHR) is a milestone document in the history of
human rights. Drafted by representatives with different legal and cultural backgrounds from
all regions of the world, it set out, for the first time, fundamental human rights to be
universally protected. The Declaration was adopted by the UN General Assembly in Paris on
10 December 1948 during its 183rd plenary meeting.
"""
_HOMEPAGE = "https://unicode.org/udhr/index.html"
_LICENSE = Licenses.UNKNOWN.value
_URLS = "https://unicode.org/udhr/assemblies/udhr_txt.zip"

_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]
_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_LANGS = {
    "ace": "Aceh",
    "ban": "Bali",
    "bcl": "Bicolano, Central",
    "blt": "Tai Dam", 
    "bug": "Bugis",
    "ceb": "Cebuano",
    "cfm": "Chin, Falam",  # flm
    "cnh": "Chin, Haka",
    "ctd": "Chin, Tedim",
    "duu": "Drung", 
    "hil": "Hiligaynon", 
    "hlt": "Chin, Matu",
    "hni": "Hani",
    "hnj": "Hmong Njua",  # blu
    "ilo": "Ilocano", 
    "ind": "Indonesian",
    "jav": "Javanese",
    "jav_java": "Javanese (Javanese)",
    "khm": "Khmer",
    "kkh": "Khun", 
    "lao": "Lao",
    "lus": "Mizo",
    "mad": "Madura",
    "min": "Minangkabau", 
    "mnw": "Mon", 
    "mya": "Burmese",
    "pam": "Pampangan", 
    "shn": "Shan",
    "sun": "Sunda",
    "tdt": "Tetun Dili", 
    "tet": "Tetun", 
    "tgl": "Tagalog",
    "tha": "Thai",
    "vie": "Vietnamese",
    "war": "Waray-waray", 
    "zlm": "Malay",  # default mly_latn
}

_LOCAL=False
_LANGUAGES=["ace", "ban", "bcl", "blt", "bug", "ceb", "cfm", "cnh", "ctd", "duu", "hil", "hlt", "hni", "hnj", "ilo", "ind", "jav", "khm", "kkh", "lao", "lus", "mad", "min", "mnw", "mya", "pam", "shn", "sun", "tdt", "tet", "tgl", "tha", "vie", "war", "zlm"]

def seacrowd_config_constructor(src_lang, schema, version):
    if src_lang == "":
        raise ValueError(f"Invalid src_lang {src_lang}")

    if schema not in ["source", "seacrowd_ssp"]:
        raise ValueError(f"Invalid schema: {schema}")

    return SEACrowdConfig(
        name="udhr_{src}_{schema}".format(src=src_lang, schema=schema),
        version=datasets.Version(version),
        description="udhr {schema} schema for {src} language".format(schema=schema, src=_LANGS[src_lang]),
        schema=schema,
        subset_id="udhr_{src}".format(src=src_lang),
    )


class UDHRDataset(datasets.GeneratorBasedBuilder):
    """
    The Universal Declaration of Human Rights (UDHR) is a milestone document in the history of
    human rights. Drafted by representatives with different legal and cultural backgrounds from
    all regions of the world, it set out, for the first time, fundamental human rights to be
    universally protected. The Declaration was adopted by the UN General Assembly in Paris on
    10 December 1948 during its 183rd plenary meeting.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [seacrowd_config_constructor(lang, "source", _SOURCE_VERSION) for lang in _LANGS] + [seacrowd_config_constructor(lang, "seacrowd_ssp", _SEACROWD_VERSION) for lang in _LANGS]

    DEFAULT_CONFIG_NAME = "udhr_ind_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_ssp":
            features = schemas.self_supervised_pretraining.features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS
        data_dir = dl_manager.download_and_extract(urls)
        lang = self.config.subset_id.split("_")
        file_key=""
        if lang[1] == "zlm":
            file_key = "mly_latn"
        elif lang[1] == "cfm":
            file_key = "flm"
        elif lang[1] == "hnj":
            file_key = "blu"        
        elif lang[1] == "kkh":
            file_key = "kkh_lana"       
        elif lang[1] == "duu":
            file_key = "020"
        elif lang[1] == "tdt":
            file_key = "010"
        elif len(lang)>2 and f"{lang[1]}_{lang[2]}" == "jav_java":
            file_key = "jav_java"
        else:
            file_key = lang[1]

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, f"udhr_{file_key}.txt".format(file_key=file_key)),
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        data = []

        with open(filepath, "r") as f:
            data = [line.rstrip() for line in f.readlines()]

        if self.config.schema == "source":
            yield 0, {"id": Path(filepath).stem, "text": " ".join(data)}

        elif self.config.schema == "seacrowd_ssp":
            yield 0, {"id": Path(filepath).stem, "text": " ".join(data)}



from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{multilexnorm,
  title= {MultiLexNorm: A Shared Task on Multilingual Lexical Normalization,
  author = "van der Goot, Rob and Ramponi et al.",
  booktitle = "Proceedings of the 7th Workshop on Noisy User-generated Text (W-NUT 2021)",
  year = "2021",
  publisher = "Association for Computational Linguistics",
  address = "Punta Cana, Dominican Republic"
}
"""

_DATASETNAME = "multilexnorm"

_DESCRIPTION = """\
MULTILEXNPRM is a new benchmark dataset for multilingual lexical normalization
including 12 language variants,
we here specifically work on the Indonisian-english language.
"""

_HOMEPAGE = "https://bitbucket.org/robvanderg/multilexnorm/src/master/"

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LICENSE = "CC-BY-NC-SA 4.0"

_URLS = {
    "train": "https://bitbucket.org/robvanderg/multilexnorm/raw/e92e5b8f111fea15c7c88aebd4c058f6a1ca8d74/data/iden/train.norm",
    "validation": "https://bitbucket.org/robvanderg/multilexnorm/raw/e92e5b8f111fea15c7c88aebd4c058f6a1ca8d74/data/iden/dev.norm",
    "test": "https://bitbucket.org/robvanderg/multilexnorm/raw/e92e5b8f111fea15c7c88aebd4c058f6a1ca8d74/data/iden/test.norm",
}

_SUPPORTED_TASKS = [Tasks.MULTILEXNORM]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class MultiLexNorm(datasets.GeneratorBasedBuilder):
    """MultiLexNorm is a new benchmark dataset for lexical normalization for indonisian English language. which is the translation
     of social media text to canonical text:
    new pix      comming tomoroe
    new pictures coming  tomorrow
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="multilexnorm_source",
            version=_SOURCE_VERSION,
            description="multilexnorm source schema",
            schema="source",
            subset_id="multilexnorm",
        ),
        SEACrowdConfig(
            name="multilexnorm_seacrowd_t2t",
            version=_SEACROWD_VERSION,
            description="multilexnorm Nusantara schema",
            schema="seacrowd_t2t",
            subset_id="multilexnorm",
        ),
    ]

    DEFAULT_CONFIG_NAME = "multilexnorm_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":

            features = datasets.Features(
                {
                    "src_sent": datasets.Value("string"),
                    "id": datasets.Value("string"),
                    "norm_sent": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:

        train_path = Path(dl_manager.download_and_extract(_URLS["train"]))
        validation_path = Path(dl_manager.download_and_extract(_URLS["validation"]))
        test_path = Path(dl_manager.download_and_extract(_URLS["test"]))
        data_files = {
            "train": train_path,
            "validation": validation_path,
            "test": test_path,
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_files["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_files["test"],
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_files["validation"],
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:

        curSent = []
        print(filepath)
        if self.config.schema == "source":
            i = 0
            for line in open(filepath):
                tok = line.strip("\n").split("\t")

                if tok == [""] or tok == []:
                    ex = {"id": str(i), 
                          "src_sent": " ".join([x[0] for x in curSent]), 
                          "norm_sent": " ".join([x[1] for x in curSent])}
                    yield i, ex
                    i += 1
                    curSent = []

                else:
                    if len(tok) > 2:
                        print("erroneous input, line:\n" + line + "\n in file " + filepath + " contains more then two elements")
                    if len(tok) == 1:
                        tok.append("")
                    curSent.append(tok)

        elif self.config.schema == "seacrowd_t2t":
            i = 0
            for line in open(filepath):
                tok = line.strip("\n").split("\t")

                if tok == [""] or tok == []:
                    ex = {"id": str(i), 
                          "text_1": " ".join([x[0] for x in curSent]), 
                          "text_2": " ".join([x[1] for x in curSent]), 
                          "text_1_name": "src_sent", 
                          "text_2_name": "norm_sent"}
                    yield i, ex
                    i += 1
                    curSent = []

                else:
                    if len(tok) > 2:
                        print("erroneous input, line:\n" + line + "\n in file " + filepath + " contains more then two elements")
                    if len(tok) == 1:
                        tok.append("")
                    curSent.append(tok)




# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@misc{IndoQA,
  author = {{Jakarta Artificial Intelligence Research}}
  title = {IndoQA: Building Indonesian QA dataset},
  year = {2023}
  url = {https://huggingface.co/datasets/jakartaresearch/indoqa}
}
"""

_DATASETNAME = "indoqa"

_DESCRIPTION = """\
IndoQA is a monolingual question-answering dataset of Indonesian language (ind).
It comprises 4,413 examples with 3:1 split of training and validation sets.
The datasets consists of a context paragraph along with an associated question-answer pair.
"""

_HOMEPAGE = "https://jakartaresearch.com/"
_LICENSE = Licenses.CC_BY_ND_4_0.value

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False
_URLS = {
    _DATASETNAME: {
        "train": "https://drive.google.com/uc?id=1ND893H5x2gaPRRMJVajQ4hgqpopHoD0u",
        "validation": "https://drive.google.com/uc?id=1mq_foV72riXb1KVBirJzTFZEe7oa8f4f",
    },
}

_SUPPORTED_TASKS = [Tasks.QUESTION_ANSWERING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IndoQADataset(datasets.GeneratorBasedBuilder):
    """IndoQA: A monolingual Indonesian question-answering dataset comprises 4,413 instances of QA-pair with context."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_qa",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_qa",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "answer": datasets.Value("string"),
                    "context": datasets.Value("string"),
                    "category": datasets.Value("string"),
                    "span_start": datasets.Value("int32"),
                    "span_end": datasets.Value("int32"),
                }
            )

        elif self.config.schema == "seacrowd_qa":
            features = schemas.qa_features
            features["meta"]["span_start"] = datasets.Value("int32")
            features["meta"]["span_end"] = datasets.Value("int32")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        data_paths = dl_manager.download_and_extract(urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_paths["train"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": data_paths["validation"]},
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        with open(filepath, "r", encoding="utf-8") as file:
            datas = json.load(file)

        if self.config.schema == "source":
            for key, data in enumerate(datas):
                yield key, data

        elif self.config.schema == "seacrowd_qa":
            for key, data in enumerate(datas):
                yield key, {
                    "id": f'{data["id"]}',
                    "question_id": data["id"],
                    "document_id": "",
                    "question": data["question"],
                    "type": data["category"],
                    "choices": [],
                    "context": data["context"],
                    "answer": [data["answer"]],
                    "meta": {
                        "span_start": data["span_start"],
                        "span_end": data["span_end"],
                    },
                }



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Code-mixed sentiment analysis of Indonesian language and Javanese language
using Lexicon based approach

Nowadays mixing one language with another language either in spoken or written
communication has become a common practice for bilingual speakers in daily
conversation as well as in social media. Lexicon based approach is one of the
approaches in extracting the sentiment analysis. This study is aimed to compare
two lexicon models which are SentiNetWord and VADER in extracting the polarity
of the code-mixed sentences in Indonesian language and Javanese language. 3,963
tweets were gathered from two accounts that provide code-mixed tweets.
Pre-processing such as removing duplicates, translating to English, filter
special characters, transform lower case and filter stop words were conducted
on the tweets. Positive and negative word score from lexicon model was then
calculated using simple mathematic formula in order to classify the polarity.
By comparing with the manual labelling, the result showed that SentiNetWord
perform better than VADER in negative sentiments. However, both of the lexicon
model did not perform well in neutral and positive sentiments. On overall
performance, VADER showed better performance than SentiNetWord. This study
showed that the reason for the misclassified was that most of Indonesian
language and Javanese language consist of words that were considered as
positive in both Lexicon model.

[seacrowd_schema_name] = (text, t2t)
"""
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@article{Tho_2021,
  doi = {10.1088/1742-6596/1869/1/012084},
  url = {https://doi.org/10.1088/1742-6596/1869/1/012084},
  year = 2021,
  month = {apr},
  publisher = {{IOP} Publishing},
  volume = {1869},
  number = {1},
  pages = {012084},
  author = {C Tho and Y Heryadi and L Lukas and A Wibowo},
  title = {Code-mixed sentiment analysis of Indonesian language and Javanese language using Lexicon based approach},
  journal = {Journal of Physics: Conference Series},
  abstract = {Nowadays mixing one language with another language either in
  spoken or written communication has become a common practice for bilingual
  speakers in daily conversation as well as in social media. Lexicon based
  approach is one of the approaches in extracting the sentiment analysis. This
  study is aimed to compare two lexicon models which are SentiNetWord and VADER
  in extracting the polarity of the code-mixed sentences in Indonesian language
  and Javanese language. 3,963 tweets were gathered from two accounts that
  provide code-mixed tweets. Pre-processing such as removing duplicates,
  translating to English, filter special characters, transform lower case and
  filter stop words were conducted on the tweets. Positive and negative word
  score from lexicon model was then calculated using simple mathematic formula
  in order to classify the polarity. By comparing with the manual labelling,
  the result showed that SentiNetWord perform better than VADER in negative
  sentiments. However, both of the lexicon model did not perform well in
  neutral and positive sentiments. On overall performance, VADER showed better
  performance than SentiNetWord. This study showed that the reason for the
  misclassified was that most of Indonesian language and Javanese language
  consist of words that were considered as positive in both Lexicon model.}
}
"""

_DATASETNAME = "code_mixed_jv_id"

_DESCRIPTION = """\
Sentiment analysis and machine translation data for Javanese and Indonesian.
"""

_HOMEPAGE = "https://iopscience.iop.org/article/10.1088/1742-6596/1869/1/012084"

_LICENSE = "cc_by_3.0"

_URLS = {
    _DATASETNAME: "https://docs.google.com/spreadsheets/d/1mq2VyPEDfXl7K6p5TbRPsaefYwkuy7RQ/export?format=csv&gid=356398080",
}

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS, Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_LANGUAGES = ['jav', 'ind']
_LOCAL = False

LANGUAGES_COLUMNS = {
    "id": ("text_ind", "text_jav"),
    "jv": ("text_jav", "text_ind"),
}


class CodeMixedSenti(datasets.GeneratorBasedBuilder):
    """Code-mixed sentiment analysis for Indonesian and Javanese."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="code_mixed_jv_id_source",
            version=SOURCE_VERSION,
            description="code_mixed_jv_id source schema for Javanese and Indonesian",
            schema="source",
            subset_id="code_mixed_source",
        ),
        SEACrowdConfig(
            name="code_mixed_jv_id_jv_seacrowd_text",
            version=SEACROWD_VERSION,
            description="code_mixed_jv_id seacrowd_text schema for Javanese",
            schema="seacrowd_text",
            subset_id="code_mixed_jv",
        ),
        SEACrowdConfig(
            name="code_mixed_jv_id_id_seacrowd_text",
            version=SEACROWD_VERSION,
            description="code_mixed_jv_id seacrowd_text schema for Indonesian",
            schema="seacrowd_text",
            subset_id="code_mixed_id",
        ),
        SEACrowdConfig(
            name="code_mixed_jv_id_seacrowd_t2t",
            version=SEACROWD_VERSION,
            description="code_mixed_jv_id seacrowd_t2t schema for Javanese and Indonesian",
            schema="seacrowd_t2t",
            subset_id="code_mixed_jv_id",
        )
    ]

    DEFAULT_CONFIG_NAME = "code_mixed_id_jv_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({
                "text_jav": datasets.Value("string"),
                "text_ind": datasets.Value("string"),
                "label": datasets.Value("int32")
            })
        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(["-1", "0", "1"])
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(description=_DESCRIPTION, features=features, homepage=_HOMEPAGE, license=_LICENSE, citation=_CITATION,)

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        url = _URLS[_DATASETNAME]
        path = dl_manager.download_and_extract(url)
        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": path, "split": "train"}),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        df = pd.read_csv(filepath,
                         skiprows=1,
                         names=["text_jav", "label", "text_ind"])
        if self.config.schema == "source":
            i = 0
            for row in df.itertuples():
                ex = {"text_jav": row.text_jav, "text_ind": row.text_ind, "label": row.label}
                yield i, ex
                i += 1
        elif self.config.schema == "seacrowd_text":
            prefix_length = len(_DATASETNAME)
            start = prefix_length + 1
            end = prefix_length + 1 + 2
            language = self.config.name[start:end]
            keep_column, drop_column = LANGUAGES_COLUMNS[language]
            df = df.drop(columns=[drop_column]).rename(columns={keep_column: "text"})
            i = 0
            for row in df.itertuples():
                ex = {"id": str(i), "text": row.text, "label": str(row.label)}
                yield i, ex
                i += 1
        elif self.config.schema == "seacrowd_t2t":
            i = 0
            for row in df.itertuples():
                ex = {"id": str(i), "text_1": row.text_jav, "text_2": row.text_ind, "text_1_name": "jav", "text_2_name": "ind"}
                yield i, ex
                i += 1

if __name__ == "__main__":
    datasets.load_dataset(__file__)



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This template serves as a starting point for contributing a dataset to the SEACrowd Datahub repo.


Full documentation on writing dataset loading scripts can be found here:
https://huggingface.co/docs/datasets/add_dataset.html

To create a dataset loading script you will create a class and implement 3 methods:
  * `_info`: Establishes the schema for the dataset, and returns a datasets.DatasetInfo object.
  * `_split_generators`: Downloads and extracts data for each split (e.g. train/val/test) or associate local data with each split.
  * `_generate_examples`: Creates examples from data on disk that conform to each schema defined in `_info`.

"""
import json
import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import (DEFAULT_SEACROWD_VIEW_NAME,
                                      DEFAULT_SOURCE_VIEW_NAME, Tasks)

_CITATION = """\
@misc{MALINDO-parallel,
  title = "MALINDO-parallel",
  howpublished = "https://github.com/matbahasa/MALINDO_Parallel/blob/master/README.md",
  note = "Accessed: 2023-01-27",
}
"""

_DATASETNAME = "malindo_parallel"


_DESCRIPTION = """\
Teks ini adalah skrip video untuk Kampus Terbuka Universiti Bahasa Asing Tokyo pada tahun 2020. Tersedia parallel sentences dalam Bahasa Melayu/Indonesia dan Bahasa Jepang
"""


_HOMEPAGE = "https://github.com/matbahasa/MALINDO_Parallel/tree/master/OpenCampusTUFS"


_LANGUAGES = ["zlm", "jpn"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)


_LICENSE = "Creative Commons Attribution 4.0 (cc-by-4.0)" 


_LOCAL = False


_URLS = {
    _DATASETNAME: "https://raw.githubusercontent.com/matbahasa/MALINDO_Parallel/master/OpenCampusTUFS/OCTUFS2020.txt",
}


_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]  # example: [Tasks.TRANSLATION, Tasks.NAMED_ENTITY_RECOGNITION, Tasks.RELATION_EXTRACTION]


_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"



class MalindoParallelDataset(datasets.GeneratorBasedBuilder):
    """Data terjemahan bahasa Melayu/Indonesia"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)


    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="malindo_parallel_source",
            version=SOURCE_VERSION,
            description="malindo_parallel source schema",
            schema="source",
            subset_id="malindo_parallel",
        ),
        SEACrowdConfig(
            name="malindo_parallel_seacrowd_t2t",
            version=SEACROWD_VERSION,
            description="malindo_parallel SEACrowd schema",
            schema="seacrowd_t2t",
            subset_id="malindo_parallel",
        ),
    ]

    DEFAULT_CONFIG_NAME = "malindo_parallel_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features({"id": datasets.Value("string"), "text": datasets.Value("string")})

        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,

                gen_kwargs={
                    "filepath": data_dir,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:

        rows = []
        temp_cols = None
        with open(filepath) as file:
            while line := file.readline():
                if temp_cols is None:
                    cols = []
                    for col in line.split('\t'):
                        if len(col.strip('\n'))>0:
                            cols.append(col)
                    if len(cols) > 2:
                        correct_line = line.rstrip()
                        rows.append(correct_line)
                    else:
                        temp_cols = cols
                else:
                    temp_cols.append(line)
                    correct_line = "\t".join(temp_cols).rstrip()
                    temp_cols = None
                    rows.append(correct_line)

        if self.config.schema == "source":

            for i, row in enumerate(rows):
                t1idx = row.find("\t") + 1
                t2idx = row[t1idx:].find("\t")
                row_id = row[:t1idx]
                row_melayu = row[t1idx : t1idx + t2idx]
                row_japanese = row[t1idx + t2idx + 1 : -1]
                ex = {"id": row_id.rstrip(),
                      "text": row_melayu + "\t" + row_japanese}
                yield i, ex

        elif self.config.schema == "seacrowd_t2t":

            for i, row in enumerate(rows):
                t1idx = row.find("\t") + 1
                t2idx = row[t1idx:].find("\t")
                row_id = row[:t1idx]
                row_melayu = row[t1idx : t1idx + t2idx]
                row_japanese = row[t1idx + t2idx + 1 : -1]
                ex = {
                    "id": row_id.rstrip(),
                    "text_1": row_melayu,
                    "text_2": row_japanese,
                    "text_1_name": "zlm",
                    "text_2_name": "jpn",
                }
                yield i, ex



if __name__ == "__main__":
    datasets.load_dataset(__file__)



# coding=utf-8
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{thai-etal-2022-uit,
title = "{UIT}-{V}i{C}o{V}19{QA}: A Dataset for {COVID}-19 Community-based Question Answering on {V}ietnamese Language",
author = "Thai, Triet and Thao-Ha, Ngan Chu and Vo, Anh  and Luu, Son",
editor = "Dita, Shirley and Trillanes, Arlene and Lucas, Rochelle Irene",
booktitle = "Proceedings of the 36th Pacific Asia Conference on Language, Information and Computation",
month = oct,
year = "2022",
address = "Manila, Philippines",
publisher = "Association for Computational Linguistics",
url = "https://aclanthology.org/2022.paclic-1.88",
pages = "801--810",
}
"""
_DATASETNAME = "uit_vicov19qa"
_DESCRIPTION = """\
UIT-ViCoV19QA is the first Vietnamese community-based question answering dataset for developing question answering
systems for COVID-19. The dataset comprises 4,500 question-answer pairs collected from trusted medical sources,
with at least one answer and at most four unique paraphrased answers per question. This dataset contains 1800 questions
that have at least two answers, 700 questions have at least three answers and half of them have a maximum of four paraphrased
answers.
"""
_HOMEPAGE = "https://github.com/triet2397/UIT-ViCoV19QA"
_LANGUAGES = ["vie"]
_LICENSE = Licenses.UNKNOWN.value
_PAPER_URL = "https://aclanthology.org/2022.paclic-1.88"
_LOCAL = False
_URLS = {
    "train": {
        "1_ans": "https://raw.githubusercontent.com/triet2397/UIT-ViCoV19QA/main/dataset/1_ans/UIT-ViCoV19QA_train.csv",
        "2_ans": "https://raw.githubusercontent.com/triet2397/UIT-ViCoV19QA/main/dataset/2_ans/UIT-ViCoV19QA_train.csv",
        "3_ans": "https://raw.githubusercontent.com/triet2397/UIT-ViCoV19QA/main/dataset/3_ans/UIT-ViCoV19QA_train.csv",
        "4_ans": "https://raw.githubusercontent.com/triet2397/UIT-ViCoV19QA/main/dataset/4_ans/UIT-ViCoV19QA_train.csv",
    },
    "val": {
        "1_ans": "https://raw.githubusercontent.com/triet2397/UIT-ViCoV19QA/main/dataset/1_ans/UIT-ViCoV19QA_val.csv",
        "2_ans": "https://raw.githubusercontent.com/triet2397/UIT-ViCoV19QA/main/dataset/2_ans/UIT-ViCoV19QA_val.csv",
        "3_ans": "https://raw.githubusercontent.com/triet2397/UIT-ViCoV19QA/main/dataset/3_ans/UIT-ViCoV19QA_val.csv",
        "4_ans": "https://raw.githubusercontent.com/triet2397/UIT-ViCoV19QA/main/dataset/4_ans/UIT-ViCoV19QA_val.csv",
    },
    "test": {
        "1_ans": "https://raw.githubusercontent.com/triet2397/UIT-ViCoV19QA/main/dataset/1_ans/UIT-ViCoV19QA_test.csv",
        "2_ans": "https://raw.githubusercontent.com/triet2397/UIT-ViCoV19QA/main/dataset/2_ans/UIT-ViCoV19QA_test.csv",
        "3_ans": "https://raw.githubusercontent.com/triet2397/UIT-ViCoV19QA/main/dataset/3_ans/UIT-ViCoV19QA_test.csv",
        "4_ans": "https://raw.githubusercontent.com/triet2397/UIT-ViCoV19QA/main/dataset/4_ans/UIT-ViCoV19QA_test.csv",
    },
}
_SUPPORTED_TASKS = [Tasks.QUESTION_ANSWERING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class ViHealthQADataset(datasets.GeneratorBasedBuilder):
    """
    This is a SeaCrowed dataloader for dataset uit_vicov19qa, The dataset comprises 4,500 question-answer pairs collected from trusted medical sources,
    with at least one answer and at most four unique paraphrased answers per question.
    """

    subsets = ["1_ans", "2_ans", "3_ans", "4_ans"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema",
            schema="source", subset_id=f"{_DATASETNAME}"),

        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_qa",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_qa",
            subset_id=f"{_DATASETNAME}",
        )
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "answers": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_qa":
            features = schemas.qa_features
        else:
            raise ValueError(f"No schema matched for {self.config.schema}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        data_dir = dl_manager.download_and_extract(_URLS)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_dir["val"],
                    "split": "val",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_dir["test"],
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath: Dict, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        print(f"Generating examples for split {split}")
        sample_id = -1
        for path in filepath.values():
            raw_examples = pd.read_csv(path, na_filter=False, delimiter="|")
            for eid, exam in raw_examples.iterrows():
                sample_id += 1
                exam_id = exam[0]
                exam_quest = exam[1]
                exam_answers = exam[2:].values
                if self.config.schema == "source":
                    yield sample_id, {"id": str(exam_id),
                                      "question": exam_quest,
                                      "answers": exam_answers
                                      }

                elif self.config.schema == "seacrowd_qa":
                    yield sample_id, {"id": str(sample_id),
                                      "question_id": exam_id,
                                      "document_id": str(sample_id),
                                      "question": exam_quest,
                                      "type": None,
                                      "choices": [],
                                      "context": None,
                                      "answer": exam_answers,
                                      "meta": {}
                                      }



from pathlib import Path
from typing import List

import re
import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, DEFAULT_SOURCE_VIEW_NAME, DEFAULT_SEACROWD_VIEW_NAME

_DATASETNAME = "korpus_seacrowd"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["ind", "jav", "xdy", "bug", "sun", "mad", "bjn", "bbc", "khek", "msa", "min", "tiociu"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False
_CITATION = """\
@article{sujaini2020improving,
  title={Improving the role of language model in statistical machine translation (Indonesian-Javanese)},
  author={Sujaini, Herry},
  journal={International Journal of Electrical and Computer Engineering},
  volume={10},
  number={2},
  pages={2102},
  year={2020},
  publisher={IAES Institute of Advanced Engineering and Science}
}
"""

_DESCRIPTION = """\
This parallel corpus was collected from several studies, assignments, and thesis of 
students of the Informatics Study Program, Tanjungpura University. Some of the corpus 
are used in the translation machine from Indonesian to local languages http://nustor.untan.ac.id/cammane/. 
This corpus can be used freely for research purposes by citing the paper 
https://ijece.iaescore.com/index.php/IJECE/article/download/20046/13738.

The dataset is a combination of multiple machine translation works from the author, 
Herry Sujaini, covering Indonesian to 25 local dialects in Indonesia. Since not all 
dialects have ISO639-3 standard coding, as agreed with Pak Herry , we decided to 
group the dataset into the closest language family, i.e.: Javanese, Dayak, Buginese, 
Sundanese, Madurese, Banjar, Batak Toba, Khek, Malay, Minangkabau, and Tiociu.
"""

_HOMEPAGE = "https://github.com/herrysujaini/korpusseacrowd"
_LICENSE = "Unknown"
_URLS = {
    _DATASETNAME: "https://github.com/herrysujaini/korpusseacrowd/raw/main/korpus seacrowd.xlsx",
}
_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


"""
A collection of all the dialects are: javanese, javanese kromo, javanese ngoko, dayak ahe, 
dayak iban, dayak pesaguan, dayak taman, buginese kelolau, buginese wajo, sundanese, 
madurese, banjar, batak toba, khek pontianak, kapuas hulu, melayu kembayan, melayu ketapang, 
melayu melawi, melayu pontianak, melayu putussibau, melayu sambas, melayu sintang, padang, 
tiociu pontianak.

In this project, we group the dialects into several subsets:

Javanese (jav)   : javanese, javanese kromo, javanese ngoko
Dayak (day)      : dayak ahe, dayak iban, dayak pesaguan, dayak taman
Buginese (bug)   : buginese kelolau, buginese wajo
Sundanese (sun)  : sundanese
Madurese (mad)   : madurese
Banjar (bjn)     : banjar
Batak Toba (bbc) : batak toba
Khek (khek)      : khek pontianak, kapuas hulu
Malay (msa)      : melayu kembayan, melayu ketapang, melayu melawi, melayu pontianak, melayu putussibau, melayu sambas, melayu sintang
Minangkabau (min): padang
Tiociu (tiociu)  : tiociu pontianak
"""

Domain2Subsets = {
    "jav": ['jawa', 'jawa kromo', 'jawa ngoko'],
    "xdy": ['dayak ahe', 'dayak iban', 'dayak pesaguan', 'dayak taman'],
    "bug": ['bugis kelolao', 'bugis wajo'],
    "sun": ['sunda'],
    "mad": ['madura'],
    "bjn": ['banjar'],
    "bbc": ['Batak'],
    "khek": ['kapuas hulu', 'Khek Pontianak'],
    "msa": ['melayu kembayan', 'melayu ketapang', 'melayu melawi', 'melayu pontianak', 'melayu putussibau', 'melayu sambas', 'melayu sintang'],
    "min": ['padang'],
    "tiociu": ['Tiociu Pontianak'],
}

class KorpusNusantara(datasets.GeneratorBasedBuilder):
    """Bible En-Id is a machine translation dataset containing Indonesian-English parallel sentences collected from the bible.."""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"korpus_seacrowd_ind_{subset}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"Korpus_Nusantara ind2{subset} source schema",
            schema="source",
            subset_id=f"korpus_seacrowd",
        )
        for subset in _LANGUAGES[1:]
    ] + \
    [
        SEACrowdConfig(
            name=f"korpus_seacrowd_ind_{subset}_seacrowd_t2t",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"Korpus_Nusantara ind2{subset} Nusantara schema",
            schema="seacrowd_t2t",
            subset_id=f"korpus_seacrowd",
        )
        for subset in _LANGUAGES[1:]
    ] + \
    [
        SEACrowdConfig(
            name=f"korpus_seacrowd_{subset}_ind_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"Korpus_Nusantara {subset}2ind source schema",
            schema="source",
            subset_id=f"korpus_seacrowd",
        )
        for subset in _LANGUAGES[1:]
    ] + \
    [
        SEACrowdConfig(
            name=f"korpus_seacrowd_{subset}_ind_seacrowd_t2t",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"Korpus_Nusantara {subset}2ind Nusantara schema",
            schema="seacrowd_t2t",
            subset_id=f"korpus_seacrowd",
        )
        for subset in _LANGUAGES[1:]
    ]

    DEFAULT_CONFIG_NAME = "korpus_seacrowd_jav_ind_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features({"id": datasets.Value("string"), "text": datasets.Value("string"), "label": datasets.Value("string")})
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        # Dataset does not have predetermined split, putting all as TRAIN
        urls = _URLS[_DATASETNAME]
        base_dir = Path(dl_manager.download(urls))
        data_files = {"train": base_dir}

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_files["train"],
                },
            ),
        ]
    
    def _merge_subsets(self, dfs, subsets, revert=False):
        if not subsets: return None
        df = None
        for subset in subsets:
            sub_df = dfs[subset]
            orig_columns = sub_df.columns.tolist()
            sub_df.columns = ["label", "text"]+orig_columns[2:] if revert else ["text", "label"]+orig_columns[2:]
            if df is None:
                df = sub_df
            else:
                df = pd.concat([df, sub_df], axis=0, sort=False)
        return df
        
    def get_domain_data(self, dfs):
        domain = self.config.name
        matched_domain = re.findall(r"korpus_seacrowd_.*?_.*?_", domain)
        
        assert len(matched_domain) == 1
        domain = matched_domain[0][:-1].replace("korpus_seacrowd_", "").split("_")
        src_lang, tgt_lang = domain[0], domain[1]
        
        subsets = Domain2Subsets.get(src_lang if src_lang != "ind" else tgt_lang, None)
        return src_lang, tgt_lang, self._merge_subsets(dfs, subsets, revert=(src_lang != "ind"))

    def _generate_examples(self, filepath: Path):
        """Yields examples as (key, example) tuples."""
        dfs = pd.read_excel(filepath, sheet_name=None, header=None)
        src_lang, tgt_lang, df = self.get_domain_data((dfs))
        
        if self.config.schema == "source":
            for idx, row in enumerate(df.itertuples()):
                ex = {
                    "id": str(idx),
                    "text": row.text,
                    "label": row.label,
                }
                yield idx, ex
                
        elif self.config.schema == "seacrowd_t2t":
            for idx, row in enumerate(df.itertuples()):
                ex = {
                    "id": str(idx),
                    "text_1": row.text,
                    "text_2": row.label,
                    "text_1_name": src_lang,
                    "text_2_name": tgt_lang,
                }
                yield idx, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")
